/*
 * This Software (C) Copyright David Goodwin, 2008, 2009.
 * 
 *   This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package iasel.gui;

import java.io.File;
import java.io.FileFilter;
import javax.imageio.ImageIO;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.ListSelectionModel;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.table.DefaultTableModel;
import org.jdesktop.application.Action;

/**
 *
 * @author  David
 */
public class WorkQueue extends javax.swing.JFrame {
    
    private DefaultTableModel dtm;
    private int inProgressIndex = -1;
    private JCheckBoxMenuItem jcbmi;
    
    /** Creates new form WorkQueue */
    public WorkQueue(JCheckBoxMenuItem jcbmi) {
        initComponents();
        dtm = (DefaultTableModel)queueTable.getModel();
        ListSelectionModel lsm = queueTable.getSelectionModel();
        lsm.setSelectionInterval(0,0);
        lsm.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        this.jcbmi = jcbmi;
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        queueTable = new javax.swing.JTable();
        jToolBar1 = new javax.swing.JToolBar();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();
        jSeparator1 = new javax.swing.JToolBar.Separator();
        jButton3 = new javax.swing.JButton();

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(iasel.Iasel.class).getContext().getResourceMap(WorkQueue.class);
        setTitle(resourceMap.getString("Form.title")); // NOI18N
        setName("Form"); // NOI18N
        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentHidden(java.awt.event.ComponentEvent evt) {
                formComponentHidden(evt);
            }
        });

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        queueTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Filename", "Done"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.Boolean.class
            };
            boolean[] canEdit = new boolean [] {
                false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        queueTable.setName("queueTable"); // NOI18N
        jScrollPane1.setViewportView(queueTable);
        queueTable.getColumnModel().getColumn(1).setMaxWidth(100);

        getContentPane().add(jScrollPane1, java.awt.BorderLayout.CENTER);

        jToolBar1.setRollover(true);
        jToolBar1.setName("jToolBar1"); // NOI18N

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(iasel.Iasel.class).getContext().getActionMap(WorkQueue.class, this);
        jButton1.setAction(actionMap.get("addFile")); // NOI18N
        jButton1.setText(resourceMap.getString("jButton1.text")); // NOI18N
        jButton1.setFocusable(false);
        jButton1.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton1.setName("jButton1"); // NOI18N
        jButton1.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jToolBar1.add(jButton1);

        jButton2.setAction(actionMap.get("addDirectory")); // NOI18N
        jButton2.setText(resourceMap.getString("jButton2.text")); // NOI18N
        jButton2.setFocusable(false);
        jButton2.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton2.setName("jButton2"); // NOI18N
        jButton2.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jToolBar1.add(jButton2);

        jSeparator1.setName("jSeparator1"); // NOI18N
        jToolBar1.add(jSeparator1);

        jButton3.setAction(actionMap.get("clearQueue")); // NOI18N
        jButton3.setText(resourceMap.getString("jButton3.text")); // NOI18N
        jButton3.setFocusable(false);
        jButton3.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton3.setName("jButton3"); // NOI18N
        jButton3.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jToolBar1.add(jButton3);

        getContentPane().add(jToolBar1, java.awt.BorderLayout.PAGE_START);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void formComponentHidden(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_formComponentHidden
         jcbmi.setState(false);
    }//GEN-LAST:event_formComponentHidden

    /** Adds one file to the end of the list.
     */
    @Action
    public void addFile() {
        JFileChooser jfc = new JFileChooser();
                FileNameExtensionFilter filter = new 
                FileNameExtensionFilter("All Supported Image Files", 
                ImageIO.getReaderFileSuffixes());
                
        jfc.setFileFilter(filter);
        jfc.setDialogTitle("Open File");
        int returnVal = jfc.showOpenDialog(jfc);
       
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            /* Load the file */
            File file = jfc.getSelectedFile();
            
            addFile(file.getAbsolutePath());
        }
        else {
            // The user aborted. Do nothing.
        }
    }
    
    public void addFile(String filename) {
        Object[] data = new Object[2];
        data[0] = filename;
        data[1] = false;
        dtm.addRow(data);
    }

    /** Adds all the files in the directory to the end of the list.
     */
    @Action
    public void addDirectory() {
        JFileChooser jdc = new JFileChooser();
        jdc.setDialogTitle("Select Directory");
        jdc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        
        int returnVal = jdc.showOpenDialog(this);
        
        java.io.FileFilter ff = new java.io.FileFilter() {

            javax.swing.filechooser.FileNameExtensionFilter filter = new 
                    FileNameExtensionFilter("All Supported Image Files", 
                    ImageIO.getReaderFileSuffixes());                 
            
            public boolean accept (File f) {
                return filter.accept(f);
            }
        };

        
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File cf = jdc.getSelectedFile();
            if (cf.isDirectory()) {
                File[] files = cf.listFiles(ff);
                        //cf.listFiles(filter);
                
                for (File f:files)
                    if (f.isFile())
                        addFile(f.getAbsolutePath());
                    
            }
        }
        
    }
    
    /** Returns the next file to work on and marks it as in progress.
     * 
     * @return The next filename or NULL if the queue is empty.
     */
    public String workOnNext() {
        // If the queue is empty, we dont have anything to work on.
        if (isEmpty()) return null;
        String filename = "";
        
        // Mark the file as finished if it hasnt been already.
        finishFile();
        
        // Move on to the next file:
        inProgressIndex++;
        
        // Get its filename
        filename = (String)dtm.getValueAt(inProgressIndex, 0);
        
        
        System.out.println("WQ: Working on file " + filename);
        
        // And return it.
        return filename;
    }
    
    /** Marks the currently in-progress file as complete.
     */
    public void finishFile() {
        // If nothing is in progress, return.
        if (inProgressIndex == -1) return;
        
        // Mark the current item as complete.
        dtm.setValueAt(true, inProgressIndex, 1);       
        
    }
    
    /** Returns true if the work queue is empty, false otherwise.
     * 
     * @return True if the work queue is empty, false otherwise.
     */
    public boolean isEmpty() {
        // Get the row count
        int rc = dtm.getRowCount();
        
        // Clearly, if there are no rows then there are no rows.
        if (rc == 0) return true;
        
        // If the item pointer is higher than the row count then we have
        // completed the last item and there are no more.
        if (rc < inProgressIndex) return true;
        
        return false;
    }

    @Action
    public void clearQueue() {
        inProgressIndex = -1;
        int j = dtm.getRowCount();
        for (int i = 0; i < j; i++)
            dtm.removeRow(0);
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JToolBar.Separator jSeparator1;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JTable queueTable;
    // End of variables declaration//GEN-END:variables
    
}
